@@ -113,7 +113,7 @@ void BluetoothDevicesDelegate::paint(QPainter *painter, const QStyleOptionViewIt
     painter->setFont(f);
     painter->drawText(r, Qt::AlignLeft | Qt::AlignTop, index.data(BluezQt::DevicesModel::FriendlyNameRole).toString());
     painter->restore();
-    painter->drawText(r, Qt::AlignLeft | Qt::AlignBottom, deviceTypeString(index.data(BluezQt::DevicesModel::DeviceTypeRole).toInt()));
+    painter->drawText(r, Qt::AlignLeft | Qt::AlignBottom, deviceTypeString(index.data(BluezQt::DevicesModel::TypeRole).toInt()));
 
     // Draw state
     r = option.rect;
@@ -166,7 +166,7 @@ QString BluetoothDevicesDelegate::deviceTypeString(int type) const
             return i18nc("This device is a Headset", "Headset");
         case BluezQt::Device::Headphones:
             return i18nc("This device are Headphones", "Headphones");
-        case BluezQt::Device::OtherAudio:
+        case BluezQt::Device::AudioVideo:
             return i18nc("This device is of type Audio", "Audio");
         case BluezQt::Device::Keyboard:
             return i18nc("This device is a Keyboard", "Keyboard");
@@ -189,7 +189,6 @@ QString BluetoothDevicesDelegate::deviceTypeString(int type) const
 
 KCMBlueDevilDevices::KCMBlueDevilDevices(QWidget *parent, const QVariantList&)
     : KCModule(parent)
-    , m_enable(new QCheckBox(i18n("Enable KDE Bluetooth Integration"), this))
     , m_systemCheck(0)
     , m_deviceDetails(0)
 {
@@ -207,14 +206,9 @@ KCMBlueDevilDevices::KCMBlueDevilDevices(QWidget *parent, const QVariantList&)
     ab->addAuthor(QStringLiteral("Rafael Fernández López"), i18n("Previous Developer and Maintainer"), QStringLiteral("ereslibre@kde.org"));
 
     setAboutData(ab);
+    setButtons(Apply);
 
     QVBoxLayout *layout = new QVBoxLayout;
-    layout->addWidget(m_enable);
-
-    m_enable->setObjectName(QStringLiteral("kcfg_enableGlobalBluetooth"));
-    addConfig(GlobalSettings::self(), this);
-
-    m_isEnabled = m_enable->isChecked();
 
     m_devices = new QListView(this);
     m_devices->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
@@ -260,19 +254,6 @@ KCMBlueDevilDevices::KCMBlueDevilDevices(QWidget *parent, const QVariantList&)
     connect(job, &BluezQt::InitManagerJob::result, this, &KCMBlueDevilDevices::initJobResult);
 }
 
-void KCMBlueDevilDevices::save()
-{
-    KCModule::save();
-    if (!m_isEnabled && m_enable->isChecked()) {
-        m_systemCheck->kded()->setModuleAutoloading(QStringLiteral("bluedevil"), true);
-        m_systemCheck->kded()->loadModule(QStringLiteral("bluedevil"));
-    } else if (m_isEnabled && !m_enable->isChecked()) {
-        m_systemCheck->kded()->setModuleAutoloading(QStringLiteral("bluedevil"), false);
-        m_systemCheck->kded()->unloadModule(QStringLiteral("bluedevil"));
-    }
-    m_isEnabled = m_enable->isChecked();
-}
-
 void KCMBlueDevilDevices::initJobResult(BluezQt::InitManagerJob *job)
 {
     if (job->error()) {
@@ -349,14 +330,14 @@ void KCMBlueDevilDevices::removeDevice()
 void KCMBlueDevilDevices::connectDevice()
 {
     BluezQt::DevicePtr device = m_devicesModel->device(m_devices->currentIndex());
-    device->connectDevice();
+    device->connectToDevice();
 }
 
 void KCMBlueDevilDevices::disconnectDevice()
 {
     m_disconnectDevice->setEnabled(false);
     BluezQt::DevicePtr device = m_devicesModel->device(m_devices->currentIndex());
-    device->disconnectDevice();
+    device->disconnectFromDevice();
 }
 
 void KCMBlueDevilDevices::launchWizard()
