@@ -22,8 +22,8 @@
 #include "../wizardagent.h"
 #include "debug_p.h"
 
-#include <QRegExpValidator>
 #include <QSortFilterProxyModel>
+#include <QRegularExpressionValidator>
 
 #include <KIconLoader>
 #include <KMessageWidget>
@@ -31,6 +31,7 @@
 #include <KPixmapSequence>
 #include <KPixmapSequenceOverlayPainter>
 
+#include <BluezQt/Manager>
 #include <BluezQt/Device>
 #include <BluezQt/Adapter>
 #include <BluezQt/DevicesModel>
@@ -79,27 +80,21 @@ QVariant DevicesProxyModel::data(const QModelIndex &index, int role) const
 
 bool DevicesProxyModel::lessThan(const QModelIndex &left, const QModelIndex &right) const
 {
-    // Move non-paired devices to the top
+    qint16 leftRssi = left.data(BluezQt::DevicesModel::RssiRole).toInt();
+    qint16 rightRssi = right.data(BluezQt::DevicesModel::RssiRole).toInt();
 
-    bool leftPaired = left.data(BluezQt::DevicesModel::PairedRole).toBool();
-    bool rightPaired = right.data(BluezQt::DevicesModel::PairedRole).toBool();
-
-    if (leftPaired > rightPaired) {
-        return true;
-    } else if (leftPaired < rightPaired) {
-        return false;
-    }
-
-    const QString &leftName = left.data(BluezQt::DevicesModel::FriendlyNameRole).toString();
-    const QString &rightName = right.data(BluezQt::DevicesModel::FriendlyNameRole).toString();
-
-    return QString::localeAwareCompare(leftName, rightName) > 0;
+    return leftRssi < rightRssi;
 }
 
 bool DevicesProxyModel::filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const
 {
     QModelIndex index = sourceModel()->index(sourceRow, 0, sourceParent);
 
+    bool devicePaired = index.data(BluezQt::DevicesModel::PairedRole).toBool();
+    if (devicePaired) {
+        return false;
+    }
+
     bool adapterPowered = index.data(BluezQt::DevicesModel::AdapterPoweredRole).toBool();
     bool adapterPairable = index.data(BluezQt::DevicesModel::AdapterPairableRole).toBool();
     return adapterPowered && adapterPairable;
@@ -132,16 +127,38 @@ DiscoverPage::DiscoverPage(BlueWizard *parent)
     connect(deviceView->selectionModel(), &QItemSelectionModel::currentChanged, this, &DiscoverPage::indexSelected);
 }
 
-void DiscoverPage::startDiscovery()
+void DiscoverPage::initializePage()
 {
+    qCDebug(WIZARD) << "Initialize Discover Page";
+
+    QList <QWizard::WizardButton> list;
+    list << QWizard::Stretch;
+    list << QWizard::NextButton;
+    list << QWizard::CancelButton;
+    m_wizard->setButtonLayout(list);
+
+    QRegularExpression rx(QStringLiteral("[0-9]{1,16}"));
+    pinText->setValidator(new QRegularExpressionValidator(rx, this));
+
+    connect(manualPin, &QCheckBox::toggled, pinText, &QLineEdit::setEnabled);
+    connect(manualPin, &QCheckBox::toggled, this, &DiscoverPage::completeChanged);
+    connect(pinText, &QLineEdit::textChanged, this, &DiscoverPage::completeChanged);
+
     m_manager = m_wizard->manager();
 
-    BluezQt::AdapterPtr adapter = m_manager->usableAdapter();
-    if (adapter && !adapter->isDiscovering()) {
-        adapter->startDiscovery();
+    m_adapter = m_manager->usableAdapter();
+    if (m_adapter && !m_adapter->isDiscovering()) {
+        qCDebug(WIZARD) << "Starting scanning";
+        m_adapter->startDiscovery();
     }
 
-    m_model->setDevicesModel(new BluezQt::DevicesModel(m_manager, this));
+    if (!m_model->sourceModel()) {
+        m_model->setDevicesModel(new BluezQt::DevicesModel(m_manager, this));
+    }
+
+    deviceView->setCurrentIndex(QModelIndex());
+    manualPin->setChecked(false);
+    pinText->clear();
 
     checkAdapters();
     connect(m_manager, &BluezQt::Manager::adapterAdded, this, &DiscoverPage::checkAdapters);
@@ -150,29 +167,9 @@ void DiscoverPage::startDiscovery()
     connect(m_manager, &BluezQt::Manager::usableAdapterChanged, this, &DiscoverPage::usableAdapterChanged);
 }
 
-void DiscoverPage::initializePage()
-{
-    qCDebug(WIZARD) << "Initialize Page";
-
-    QList <QWizard::WizardButton> list;
-    list << QWizard::Stretch;
-    list << QWizard::NextButton;
-    list << QWizard::CancelButton;
-    m_wizard->setButtonLayout(list);
-
-    QRegExp rx(QStringLiteral("[0-9]{0,9}"));
-    QRegExpValidator *validator = new QRegExpValidator(rx);
-    pinText->setValidator(validator);
-
-    connect(manualPin, &QCheckBox::toggled, pinText, &QLineEdit::setEnabled);
-    connect(manualPin, &QCheckBox::toggled, this, &DiscoverPage::completeChanged);
-    connect(pinText, &QLineEdit::textChanged, m_wizard, &BlueWizard::setPin);
-    connect(pinText, &QLineEdit::textChanged, this, &DiscoverPage::completeChanged);
-}
-
 bool DiscoverPage::isComplete() const
 {
-    if (!m_wizard->device()) {
+    if (!m_wizard->device() || m_wizard->device()->isPaired()) {
         return false;
     }
 
@@ -199,54 +196,41 @@ int DiscoverPage::nextId() const
 
     BluezQt::DevicePtr device = m_wizard->device();
 
-    qCDebug(WIZARD) << "Stopping scanning";
-    device->adapter()->stopDiscovery();
-
     if (device->isPaired()) {
-        qCDebug(WIZARD) << "Device is paired, skipping to connect";
-        return BlueWizard::Connect;
+        return BlueWizard::Discover;
     }
 
-    QString pin;
+    if (m_adapter && m_adapter->isDiscovering()) {
+        qCDebug(WIZARD) << "Stopping scanning";
+        m_adapter->stopDiscovery();
+    }
+
+    QString pin = m_wizard->agent()->getPin(device);
+
     if (manualPin->isChecked()) {
-        pin = m_wizard->pin();
+        pin = pinText->text();
         m_wizard->agent()->setPin(pin);
-    } else {
-        pin = m_wizard->agent()->getPin(device);
     }
 
-    qCDebug(WIZARD) << "Class: " << device->deviceType();
+    qCDebug(WIZARD) << "Device type: " << BluezQt::Device::typeToString(device->type());
     qCDebug(WIZARD) << "Legacy: " << device->hasLegacyPairing();
     qCDebug(WIZARD) << "From DB: " << m_wizard->agent()->isFromDatabase();
-    qCDebug(WIZARD) << "PIN: " << m_wizard->agent()->pin();
+    qCDebug(WIZARD) << "PIN: " << pin;
 
-    // If keyboard no matter what, we go to the keyboard page
-    if (device->deviceType() == BluezQt::Device::Keyboard) {
-        qCDebug(WIZARD) << "Keyboard Pairing";
-        return BlueWizard::KeyboardPairing;
-    }
-
-    // No legacy pairing and no database pin means secure pairing
-    if (!device->hasLegacyPairing() && !m_wizard->agent()->isFromDatabase()) {
-        qCDebug(WIZARD) << "Secure Pairing";
-        return BlueWizard::SSPPairing;
-    }
-
-    // NULL pin means that no pairing is required
+    // NULL pin means that we should only connect to device
     if (pin == QLatin1String("NULL")) {
-        qCDebug(WIZARD) << "No Pairing";
         return BlueWizard::Connect;
     }
 
-    if (m_wizard->agent()->isFromDatabase()) {
-        return BlueWizard::LegacyPairingDatabase;
-    } else {
-        return BlueWizard::LegacyPairing;
-    }
+    return BlueWizard::Pairing;
 }
 
 void DiscoverPage::indexSelected(const QModelIndex &index)
 {
+    if (m_wizard->currentId() != BlueWizard::Discover) {
+        return;
+    }
+
     BluezQt::DevicePtr device = m_model->device(index);
     m_wizard->setDevice(device);
 
@@ -255,8 +239,10 @@ void DiscoverPage::indexSelected(const QModelIndex &index)
 
 void DiscoverPage::usableAdapterChanged(BluezQt::AdapterPtr adapter)
 {
-    if (adapter && !adapter->isDiscovering()) {
-        adapter->startDiscovery();
+    m_adapter = adapter;
+
+    if (m_adapter && !m_adapter->isDiscovering()) {
+        m_adapter->startDiscovery();
     }
 
     checkAdapters();
